from __future__ import absolute_import, division, print_function, unicode_literals

import csv
import inspect

#  All possible class names must be inserted into the globals collection.
#  If there is a better way of doing this, please suggest!
from amaascore.assets.asset import Asset
from amaascore.assets.bond import BondCorporate, BondGovernment, BondMortgage
from amaascore.assets.bond_option import BondOption
from amaascore.assets.currency import Currency
from amaascore.assets.derivative import Derivative
from amaascore.assets.equity import Equity
from amaascore.assets.foreign_exchange import ForeignExchange, NonDeliverableForward


def json_to_asset(json_asset):
    # Iterate through the asset children, converting the various JSON attributes into the relevant class type
    for (collection_name, clazz) in Asset.children().items():
        children = json_asset.pop(collection_name, {})
        collection = {}
        for (child_type, child_json) in children.items():
            # Handle the case where there are multiple children for a given type - e.g. links
            if isinstance(child_json, list):
                child = set()
                for child_json_in_list in child_json:
                    child.add(clazz(**child_json_in_list))
            else:
                child = clazz(**child_json)
            collection[child_type] = child
        json_asset[collection_name] = collection
    clazz = globals().get(json_asset.get('asset_type'))
    args = inspect.getargspec(clazz.__init__)
    # Some fields are always added in, even though they're not explicitly part of the constructor
    clazz_args = args.args + clazz.amaas_model_attributes()
    # is not None is important so it includes zeros and False
    constructor_dict = {arg: json_asset.get(arg) for arg in clazz_args
                        if json_asset.get(arg) is not None and arg != 'self'}
    asset = clazz(**constructor_dict)
    return asset


def csv_filename_to_assets(filename):
    with open(filename, 'rb') as f:
        assets = csv_stream_to_assets(f)
    return assets


def csv_stream_to_assets(stream):
    reader = csv.DictReader(stream)
    assets = []
    for row in reader:
        assets.append(json_to_asset(row))
    return assets


def assets_to_csv(assets, filename):
    with open(filename, 'w') as csvfile:
        assets_to_csv_stream(assets=assets, stream=csvfile)


def assets_to_csv_stream(assets, stream):
    if not assets:
        return
    asset_dicts = []
    for asset in assets:
        asset_dict = asset.to_json()
        # FOR NOW - remove all children
        [asset_dict.pop(child, None) for child in asset.children().keys()]
        asset_dicts.append(asset_dict)
    fieldnames = asset_dicts[0].keys()
    writer = csv.DictWriter(stream, fieldnames=fieldnames)
    writer.writeheader()
    writer.writerows(asset_dicts)



from __future__ import absolute_import, division, print_function, unicode_literals

import datetime
import uuid

from amaascore.core.amaas_model import AMaaSModel


class Book(AMaaSModel):

    @staticmethod
    def non_interface_attributes():
        return ['positions']

    def __init__(self, asset_manager_id, book_id=None, book_status='Active', owner_id=None, party_id=None,
                 close_time=datetime.timedelta(hours=18), timezone='UTC', description='', positions=None,
                 *args, **kwargs):
        self.asset_manager_id = asset_manager_id
        self.book_id = book_id or uuid.uuid4().hex
        self.book_status = book_status
        self.owner_id = owner_id
        self.party_id = party_id
        self.close_time = close_time
        self.timezone = timezone
        self.description = description
        self.positions = positions
        super(Book, self).__init__(*args, **kwargs)

    def positions_by_asset(self):
        """
        A dictionary of Position objects keyed by asset_id.
        :return:
        """
        return {position.asset_id: position for position in self.positions}



from __future__ import absolute_import, division, print_function, unicode_literals

from datetime import datetime, date
import pytz
import random

from amaascore.market_data.eod_price import EODPrice
from amaascore.market_data.fx_rate import FXRate
from amaascore.market_data.quote import Quote
from amaascore.tools.helpers import random_decimal, random_string


def generate_eod_price(asset_manager_id=0, asset_id=None, business_date=date.today(), price=None):
    eod_price = EODPrice(asset_manager_id=asset_manager_id,
                         asset_id=asset_id or random_string(10),
                         business_date=business_date,
                         price=price or random_decimal())
    return eod_price


def generate_fx_rate(asset_manager_id=0, asset_id=None, rate_timestamp=None, rate=None, business_date=None,
                     rate_type='EOD'):
    now = datetime.utcnow().replace(tzinfo=pytz.utc, microsecond=0)
    rate_timestamp = rate_timestamp or now
    business_date = business_date or rate_timestamp.date() if type(rate_timestamp) == datetime else rate_timestamp
    fx_rate = FXRate(asset_manager_id=asset_manager_id,
                     asset_id=asset_id or random.choice(['USDJPY', 'USDSGD', 'EURUSD']),
                     business_date=business_date,
                     rate_timestamp=rate_timestamp,
                     rate=rate or random_decimal(),
                     rate_type=rate_type)
    return fx_rate


def generate_quote(asset_manager_id=0, asset_id=None, bid=None, ask=None):
    quote = Quote(asset_manager_id=asset_manager_id,
                  asset_id=asset_id or random_string(10),
                  bid=bid or random_decimal(),
                  ask=ask or random_decimal(),
                  quote_datetime=datetime.utcnow())
    return quote




from datetime import date
from dateutil.parser import parse
from decimal import Decimal

from amaascore.assets.asset import Asset


class BondBase(Asset):

    def __init__(self, asset_manager_id, asset_id, maturity_date, coupon, par, pay_frequency, asset_issuer_id,
                 asset_status, description, country_id, venue_id, issue_date, defaulted, links, references,
                 *args, **kwargs):
        self.asset_class = 'Bond'
        self.coupon = coupon
        self.par = par
        self.pay_frequency = pay_frequency
        self.defaulted = defaulted
        super(BondBase, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id, fungible=True,
                                       asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                       description=description, country_id=country_id, venue_id=venue_id,
                                       issue_date=issue_date, maturity_date=maturity_date, links=links,
                                       references=references,
                                       *args, **kwargs)

    @property
    def coupon(self):
        if hasattr(self, '_coupon'):
            return self._coupon

    @coupon.setter
    def coupon(self, coupon):
        """
        The coupon paid out by the bond.  Represented as a fraction of 1 (e.g. 0.05 is 5%).
        :param coupon:
        :return:
        """
        if coupon is not None:
            self._coupon = Decimal(coupon)

    @property
    def par(self):
        if hasattr(self, '_par'):
            return self._par

    @par.setter
    def par(self, par):
        """
        The face value of each bond.
        Force this to be Decimal
        :param par:
        :return:
        """
        if par is not None:
            self._par = Decimal(par)


class BondGovernment(BondBase):

    def __init__(self, asset_manager_id, asset_id, coupon, par, pay_frequency, defaulted=False, asset_issuer_id=None,
                 maturity_date=date.max, asset_status='Active', description='', country_id=None, venue_id=None,
                 issue_date=date.min, links={}, references={}, *args, **kwargs):
        super(BondGovernment, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id,
                                             asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                             description=description, country_id=country_id, venue_id=venue_id,
                                             maturity_date=maturity_date, links=links, references=references,
                                             coupon=coupon, par=par, issue_date=issue_date, pay_frequency=pay_frequency,
                                             defaulted=defaulted, *args, **kwargs)


class BondCorporate(BondBase):

    def __init__(self, asset_manager_id, asset_id, coupon, par, pay_frequency, defaulted=False, asset_issuer_id=None,
                 maturity_date=date.max, asset_status='Active', description='', country_id=None, venue_id=None,
                 issue_date=date.min, links={}, references={}, *args, **kwargs):
        super(BondCorporate, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id,
                                            asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                            description=description, country_id=country_id, venue_id=venue_id,
                                            maturity_date=maturity_date, links=links, references=references,
                                            coupon=coupon, par=par, issue_date=issue_date, pay_frequency=pay_frequency,
                                            defaulted=defaulted, *args, **kwargs)


class BondMortgage(BondBase):

    def __init__(self, asset_manager_id, asset_id, coupon, par, pay_frequency, defaulted=False, asset_issuer_id=None,
                 maturity_date=date.max, asset_status='Active', description='', country_id=None, venue_id=None,
                 issue_date=date.min, links={}, references={}, *args, **kwargs):
        super(BondMortgage, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id,
                                           asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                           description=description, country_id=country_id, venue_id=venue_id,
                                           maturity_date=maturity_date, links=links, references=references,
                                           coupon=coupon, par=par, issue_date=issue_date, pay_frequency=pay_frequency,
                                           defaulted=defaulted, *args, **kwargs)



from __future__ import absolute_import, division, print_function, unicode_literals

import csv
import inspect

#  All possible class names must be inserted into the globals collection.
#  If there is a better way of doing this, please suggest!
from amaascore.corporate_actions.corporate_action import CorporateAction
from amaascore.corporate_actions.dividend import Dividend
from amaascore.corporate_actions.split import Split


def json_to_corporate_action(json_corporate_action):
    clazz = globals().get(json_corporate_action.get('corporate_action_type'))
    args = inspect.getargspec(clazz.__init__)
    # Some fields are always added in, even though they're not explicitly part of the constructor
    clazz_args = args.args + clazz.amaas_model_attributes()
    # is not None is important so it includes zeros and False
    constructor_dict = {arg: json_corporate_action.get(arg) for arg in clazz_args
                        if json_corporate_action.get(arg) is not None and arg != 'self'}
    corporate_action = clazz(**constructor_dict)
    return corporate_action


def csv_filename_to_corporate_actions(filename):
    with open(filename, 'rb') as f:
        corporate_actions = csv_stream_to_corporate_actions(f)
    return corporate_actions


def csv_stream_to_corporate_actions(stream):
    reader = csv.DictReader(stream)
    corporate_actions = []
    for row in reader:
        corporate_actions.append(json_to_corporate_action(row))
    return corporate_actions


def corporate_actions_to_csv(corporate_actions, filename):
    with open(filename, 'w') as csvfile:
        corporate_actions_to_csv_stream(corporate_actions=corporate_actions, stream=csvfile)


def corporate_actions_to_csv_stream(corporate_actions, stream):
    if not corporate_actions:
        return
    corporate_action_dicts = []
    for corporate_action in corporate_actions:
        corporate_action_dicts.append(corporate_action.to_json())
    fieldnames = corporate_action_dicts[0].keys()
    writer = csv.DictWriter(stream, fieldnames=fieldnames)
    writer.writeheader()
    writer.writerows(corporate_action_dicts)



from amaascore.assets.listed_derivative import ListedDerivative


class Future(ListedDerivative):

    def __init__(self, asset_manager_id, asset_id, asset_issuer_id=None, asset_status='Active', issue_date=None,
                 expiry_date=None, description='', country_id=None, venue_id=None, links={}, references={},
                 *args, **kwargs):
        super(Future, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id,
                                     asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                     description=description, country_id=country_id, venue_id=venue_id,
                                     links=links, references=references, issue_date=issue_date, expiry_date=expiry_date,
                                     *args, **kwargs)



# POTENTIALLY SUPPORT MULTIPLE ENVIRONMENTS

LOCAL = False

if LOCAL:
    ENDPOINTS = {
        'asset_managers': 'http://localhost:8000',
        'assets': 'http://localhost:8000',
        'books': 'http://localhost:8000',
        'corporate_actions': 'http://localhost:8000',
        'market_data': 'http://localhost:8000',
        'monitor': 'http://localhost:8000',
        'parties': 'http://localhost:8000',
        'transactions': 'http://localhost:8000'
    }
else:
    ENDPOINTS = {
        'asset_managers': 'https://c1hes1s60m.execute-api.ap-southeast-1.amazonaws.com/dev',
        'assets': 'https://zc6udsq1nb.execute-api.ap-southeast-1.amazonaws.com/dev',
        'books': 'https://smc367plfg.execute-api.ap-southeast-1.amazonaws.com/dev',
        'corporate_actions': 'https://basklngdyh.execute-api.ap-southeast-1.amazonaws.com/dev',
        'market_data': 'https://f0rpi7vksi.execute-api.ap-southeast-1.amazonaws.com/dev',
        'monitor': 'https://wt50nd7j7l.execute-api.ap-southeast-1.amazonaws.com/dev',
        'parties': 'https://hpihgzmxoc.execute-api.ap-southeast-1.amazonaws.com/dev',
        'transactions': 'https://1w0gb581sl.execute-api.ap-southeast-1.amazonaws.com/dev'
    }



import requests

from amaascore.asset_managers.utils import json_to_asset_manager
from amaascore.config import ENDPOINTS
from amaascore.core.interface import Interface


class AssetManagersInterface(Interface):
    """
    The interface to the Asset Managers service for reading Asset Manager information.
    """

    def __init__(self):
        endpoint = ENDPOINTS.get('asset_managers')
        super(AssetManagersInterface, self).__init__(endpoint=endpoint)

    def new(self, asset_manager):
        url = '%s/asset_managers' % self.endpoint
        response = requests.post(url, json=asset_manager.to_interface())
        if response.ok:
            return json_to_asset_manager(response.json())
        else:
            print("HANDLE THIS PROPERLY")
            print(response.content)

    def retrieve(self, asset_manager_id):
        url = '%s/asset_managers/%s' % (self.endpoint, asset_manager_id)
        response = requests.get(url)
        if response.ok:
            return json_to_asset_manager(response.json())
        else:
            print("HANDLE THIS PROPERLY")
            print(response.content)

    def deactivate(self, asset_manager_id):
        """
        Is is only possible to deactivate an asset manager if your client_id is also the client_id that was used
        to originally create the asset manager.

        :param asset_manager_id:
        :return:
        """
        url = '%s/asset_managers/%s' % (self.endpoint, asset_manager_id)
        response = requests.delete(url)
        if response.ok:
            print("DO SOMETHING?")
        else:
            print("HANDLE THIS PROPERLY")
            print(response.content)

    def search(self, asset_manager_ids=None, client_ids=None):
        search_params = {}
        # Potentially roll this into a loop through args rather than explicitly named - depends on additional validation
        if asset_manager_ids:
            search_params['asset_manager_ids'] = asset_manager_ids
        if client_ids:
            search_params['client_ids'] = client_ids
        url = self.endpoint + '/asset_managers'
        response = requests.get(url, params=search_params)
        if response.ok:
            assets = [json_to_asset_manager(json_asset_manager) for json_asset_manager in response.json()]
            return assets
        else:
            print("HANDLE THIS PROPERLY")
            print(response.content)





from __future__ import absolute_import, division, print_function, unicode_literals

from datetime import date
from dateutil.parser import parse
from decimal import Decimal
import sys

from amaascore.core.amaas_model import AMaaSModel

# This extremely ugly hack is due to the whole Python 2 vs 3 debacle.
type_check = str if sys.version_info >= (3, 0, 0) else (str, unicode)


class EODPrice(AMaaSModel):

    def __init__(self, asset_manager_id, asset_id, business_date, price, active=True):
        """

        :param asset_manager_id: The asset_manager_id who owns this price
        :param asset_id:  The asset_id for which we are providing a price
        :param business_date:  The business date for which this is the EOD price
        :param price: The price itself
        :param active: A boolean indicator of whether this price can be used or whether it has been deactivated
        """
        self.asset_manager_id = asset_manager_id
        self.asset_id = asset_id
        self.price = price
        self.business_date = business_date
        self.active = active
        super(EODPrice, self).__init__()

    @property
    def price(self):
        return self._price

    @price.setter
    def price(self, value):
        """
        Force the price to always be a decimal
        :param value:
        :return:
        """
        if value is not None:
            self._price = Decimal(value)

    @property
    def business_date(self):
        return self._business_date

    @business_date.setter
    def business_date(self, business_date):
        """
        Force the business_date to always be a date
        :param business_date:
        :return:
        """
        if business_date is not None:
            if isinstance(business_date, type_check):
                self._business_date = parse(business_date).date()
            else:
                self._business_date= business_date



from __future__ import absolute_import, division, print_function, unicode_literals

import datetime
from decimal import Decimal
import random

from amaascore.core.reference import Reference
from amaascore.transactions.children import Charge, Code, Comment, Link, Party
from amaascore.transactions.enums import TRANSACTION_ACTIONS
from amaascore.transactions.position import Position
from amaascore.transactions.transaction import Transaction
from amaascore.tools.helpers import random_string

CHARGE_TYPES = ['Tax', 'Commission']
CODE_TYPES = ['Settle Code', 'Client Classifier']
COMMENT_TYPES = ['Trader']
PARTY_TYPES = ['Prime Broker']
REFERENCE_TYPES = ['External']


def generate_common(asset_manager_id, asset_book_id, counterparty_book_id, asset_id, quantity, price, transaction_date,
                    transaction_id, transaction_action, transaction_type, transaction_status, transaction_currency,
                    settlement_currency):

    # Explicitly handle price is None (in case price is 0)
    price = Decimal(random.uniform(1.0, 1000.0)).quantize(Decimal('0.01')) if price is None else price

    common = {'asset_manager_id': asset_manager_id or random.randint(1, 1000),
              'asset_book_id': asset_book_id or random_string(8),
              'counterparty_book_id': counterparty_book_id or random_string(8),
              'asset_id': asset_id or str(random.randint(1, 1000)),
              'quantity': quantity or Decimal(random.randint(0, 5000)),
              'price': price,
              'transaction_date': transaction_date or datetime.date.today(),
              'transaction_currency': transaction_currency or random.choice(['SGD', 'USD']),
              'settlement_currency': settlement_currency or transaction_currency or random.choice(['SGD', 'USD']),
              'transaction_action': transaction_action or random.choice(list(TRANSACTION_ACTIONS)),
              'transaction_id': transaction_id,
              'transaction_status': transaction_status or 'New',
              'transaction_type': transaction_type or 'Trade'
              }

    common['settlement_date'] = (datetime.timedelta(days=2) + common['transaction_date'])
    return common


def generate_transaction(asset_manager_id=None, asset_book_id=None, counterparty_book_id=None,
                         asset_id=None, quantity=None, transaction_date=None, transaction_id=None,
                         price=None, net_affecting_charges=None, transaction_action=None, transaction_type=None,
                         transaction_status=None, transaction_currency=None, settlement_currency=None):

    common = generate_common(asset_manager_id=asset_manager_id, asset_book_id=asset_book_id,
                             counterparty_book_id=counterparty_book_id, asset_id=asset_id, quantity=quantity,
                             price=price, transaction_date=transaction_date, transaction_id=transaction_id,
                             transaction_action=transaction_action, transaction_status=transaction_status,
                             transaction_type=transaction_type, transaction_currency=transaction_currency,
                             settlement_currency=settlement_currency)

    transaction = Transaction(**common)
    charges = {charge_type: Charge(charge_value=Decimal(random.uniform(1.0, 100.0)).quantize(Decimal('0.01')),
                                   currency=random.choice(['USD', 'SGD']),
                                   net_affecting=net_affecting_charges or random.choice([True, False]))
               for charge_type in CHARGE_TYPES}

    links = {'Single': Link(linked_transaction_id=random_string(8)),
             'Multiple': {Link(linked_transaction_id=random_string(8)) for x in range(3)}}

    codes = {code_type: Code(code_value=random_string(8)) for code_type in CODE_TYPES}
    comments = {comment_type: Comment(comment_value=random_string(8)) for comment_type in COMMENT_TYPES}
    parties = {party_type: Party(party_id=random_string(8)) for party_type in PARTY_TYPES}
    references = {ref_type: Reference(reference_value=random_string(10)) for ref_type in REFERENCE_TYPES}

    transaction.charges.update(charges)
    transaction.codes.update(codes)
    transaction.comments.update(comments)
    transaction.links.update(links)
    transaction.parties.update(parties)
    transaction.references.update(references)
    return transaction


def generate_position(asset_manager_id=None, asset_book_id=None, asset_id=None, quantity=None):
    position = Position(asset_manager_id=asset_manager_id or random.randint(1, 1000),
                        asset_book_id=asset_book_id or random_string(8),
                        asset_id=asset_id or str(random.randint(1, 1000)),
                        quantity=quantity or Decimal(random.randint(1, 50000)))
    return position



from amaascore.assets.asset import Asset


class ForeignExchangeBase(Asset):
    """ This class should never be instantiated """

    def __init__(self, asset_id, asset_status, description, *args, **kwargs):
        self.asset_class = 'ForeignExchange'
        super(ForeignExchangeBase, self).__init__(asset_manager_id=0, asset_id=asset_id, fungible=True,
                                                  asset_status=asset_status, description=description,
                                                  *args, **kwargs)

    def base_currency(self):
        return self.asset_id[0:3]

    def counter_currency(self):
        return self.asset_id[3:6]


class ForeignExchange(ForeignExchangeBase):
    """
    Currently modelling spot and forward as the same, just two different dates on the transaction.  We might need to
    change that.
    """
    def __init__(self, asset_id, asset_status='Active', description='', *args, **kwargs):
        super(ForeignExchange, self).__init__(asset_id=asset_id, asset_status=asset_status, description=description,
                                              *args, **kwargs)


class NonDeliverableForward(ForeignExchangeBase):
    """
    Currently modelling spot and forward as the same, just two different dates on the transaction.  We might need to
    change that.
    """

    def __init__(self, asset_id, asset_status='Active', description='', *args, **kwargs):
        super(NonDeliverableForward, self).__init__(asset_id=asset_id, asset_status=asset_status,
                                                    description=description, *args, **kwargs)




from amaascore.parties.company import Company


class Fund(Company):

    def __init__(self, asset_manager_id, party_id, base_currency, description='', party_status='Active',
                 addresses={}, emails={}, links={},
                 references={}, *args, **kwargs):
        super(Fund, self).__init__(asset_manager_id=asset_manager_id, party_id=party_id, base_currency=base_currency,
                                   description=description, party_status=party_status,
                                   addresses=addresses, emails=emails, links=links, references=references,
                                   *args, **kwargs)



TRANSACTION_TYPES = {'Allocation', 'Block', 'Cashflow', 'Coupon', 'Dividend', 'Exercise', 'Expiry', 'Payment',
                     'Journal', 'Maturity', 'Net', 'Novation', 'Trade'}
TRANSACTION_LIFECYCLE_ACTIONS = {'Acquire', 'Remove'}
TRANSACTION_ACTIONS = {'Buy', 'Sell', 'Short Sell', 'Deliver', 'Receive'} | TRANSACTION_LIFECYCLE_ACTIONS
TRANSACTION_CANCEL_STATUSES = {'Cancelled', 'Netted', 'Novated'}
TRANSACTION_STATUSES = {'New', 'Amended', 'Superseded'} | TRANSACTION_CANCEL_STATUSES



from decimal import Decimal

from amaascore.corporate_actions.corporate_action import CorporateAction


class Dividend(CorporateAction):

    def __init__(self, asset_manager_id, corporate_action_id, record_date, dividend_rate, dividend_asset_id,
                 corporate_action_status='Open', asset_id=None, party_id=None, declared_date=None, settlement_date=None,
                 elective=False, message=None, description='', references=None, *args, **kwargs):
        self.dividend_rate = dividend_rate
        self.dividend_asset_id = dividend_asset_id
        super(Dividend, self).__init__(asset_manager_id=asset_manager_id, corporate_action_id=corporate_action_id,
                                       record_date=record_date, corporate_action_status=corporate_action_status,
                                       asset_id=asset_id, party_id=party_id, declared_date=declared_date,
                                       settlement_date=settlement_date, elective=elective, message=message,
                                       description=description, references=references, *args, **kwargs)

    @property
    def dividend_rate(self):
        if hasattr(self, '_dividend_rate'):
            return self._dividend_rate

    @dividend_rate.setter
    def dividend_rate(self, value):
        """
        The rate per share to pay out for the dividend
        :param value:
        :return:
        """
        if value:
            self._dividend_rate = Decimal(value)



from decimal import Decimal

from amaascore.assets.future import Future
from amaascore.assets.option_mixin import OptionMixin


class FutureOption(Future, OptionMixin):

    def __init__(self, asset_manager_id, asset_id, option_type, option_style, strike, underlying_asset_id,
                 asset_issuer_id=None, asset_status='Active', issue_date=None, expiry_date=None, description='',
                 country_id=None, venue_id=None, links={}, references={}, *args, **kwargs):
        self.option_type = option_type
        self.option_style = option_style
        self.strike = strike
        self.underlying_asset_id = underlying_asset_id
        super(FutureOption, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id,
                                           asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                           description=description, country_id=country_id, venue_id=venue_id,
                                           links=links, references=references, issue_date=issue_date,
                                           expiry_date=expiry_date,
                                           *args, **kwargs)



from decimal import Decimal

from amaascore.core.amaas_model import AMaaSModel


class Position(AMaaSModel):

    def __init__(self, asset_manager_id, asset_book_id, asset_id, quantity, *args, **kwargs):

        self.asset_manager_id = asset_manager_id
        self.asset_book_id = asset_book_id
        self.asset_id = asset_id
        self.quantity = quantity
        super(Position, self).__init__(*args, **kwargs)

    @property
    def quantity(self):
        return self._quantity

    @quantity.setter
    def quantity(self, value):
        """
        Force the quantity to always be a decimal
        :param value:
        :return:
        """
        self._quantity = Decimal(value)


class Interface(object):
    """
    Currently this class doesn't do anything - but I anticipate it will be needed in the future.
    """

    def __init__(self, endpoint):
        self.endpoint = endpoint



import requests

from amaascore.config import ENDPOINTS
from amaascore.core.interface import Interface


class MarketDataInterface(Interface):

    def __init__(self):
        endpoint = ENDPOINTS.get('market_data')
        super(MarketDataInterface, self).__init__(endpoint=endpoint)

    # def new(self, book):
    #     url = self.endpoint + '/books'
    #     response = requests.post(url, json=book.to_interface())
    #     if response.ok:
    #         book = json_to_book(response.json())
    #         return book
    #     else:
    #         print("HANDLE THIS PROPERLY")
    #         print(response.content)
    #
    # def amend(self, book):
    #     url = '%s/books/%s/%s' % (self.endpoint, book.asset_manager_id, book.book_id)
    #     response = requests.put(url, json=book.to_interface())
    #     if response.ok:
    #         book = json_to_book(response.json())
    #         return book
    #     else:
    #         print("HANDLE THIS PROPERLY")
    #         print(response.content)
    #
    # def retrieve(self, asset_manager_id, book_id):
    #     url = '%s/books/%s/%s' % (self.endpoint, asset_manager_id, book_id)
    #     response = requests.get(url)
    #     if response.ok:
    #         return json_to_book(response.json())
    #     else:
    #         print("HANDLE THIS PROPERLY")
    #         print(response.content)
    #
    # def retire(self, asset_manager_id, book_id):
    #     url = '%s/books/%s/%s' % (self.endpoint, asset_manager_id, book_id)
    #     response = requests.delete(url)
    #     if response.ok:
    #         print("DO SOMETHING?")
    #     else:
    #         print("HANDLE THIS PROPERLY")
    #         print(response.content)
    #
    # def search(self, asset_manager_ids=None, book_ids=None):
    #     search_params = {}
    #     # Potentially roll this into a loop through args rather than explicitly named - depends on additional validation
    #     if asset_manager_ids:
    #         search_params['asset_manager_ids'] = asset_manager_ids
    #     if book_ids:
    #         search_params['book_ids'] = book_ids
    #     url = self.endpoint + '/books'
    #     response = requests.get(url, params=search_params)
    #     if response.ok:
    #         books = [json_to_book(json_book) for json_book in response.json()]
    #         return books
    #     else:
    #         print("HANDLE THIS PROPERLY")
    #         print(response.content)



from __future__ import absolute_import, division, print_function, unicode_literals

from datetime import timedelta
from dateutil.parser import parse

from amaascore.core.amaas_model import AMaaSModel


class AssetManager(AMaaSModel):

    def __init__(self, asset_manager_type, asset_manager_id=None, asset_manager_status='Active', party_id=None,
                 default_book_owner_id=None, default_timezone='UTC', default_book_close_time=timedelta(hours=18),
                 *args, **kwargs):
        self.asset_manager_id = asset_manager_id
        self.asset_manager_type = asset_manager_type
        self.asset_manager_status = asset_manager_status
        self.party_id = party_id
        self.default_book_owner_id = default_book_owner_id
        self.default_timezone = default_timezone
        self.default_book_close_time = default_book_close_time
        super(AssetManager, self).__init__(*args, **kwargs)



from decimal import Decimal

from amaascore.core.amaas_model import AMaaSModel


class Charge(AMaaSModel):

    def __init__(self, charge_value, currency, active=True, net_affecting=True, version=1, *args, **kwargs):
        self.charge_value = charge_value
        self.currency = currency
        self.active = active
        self.net_affecting = net_affecting
        self.version = version
        super(Charge, self).__init__(*args, **kwargs)

    @property
    def charge_value(self):
        return self._charge_value

    @charge_value.setter
    def charge_value(self, value):
        """
        Force the charge_value to always be a decimal
        :param value:
        :return:
        """
        self._charge_value = Decimal(value)


class Code(AMaaSModel):

    def __init__(self, code_value, active=True, version=1, *args, **kwargs):
        self.code_value = code_value
        self.active = active
        self.version = version
        super(Code, self).__init__(*args, **kwargs)


class Comment(AMaaSModel):

    def __init__(self, comment_value, active=True, version=1, *args, **kwargs):
        self.comment_value = comment_value
        self.active = active
        self.version = version
        super(Comment, self).__init__(*args, **kwargs)


class Link(AMaaSModel):

    def __init__(self, linked_transaction_id, active=True, version=1, *args, **kwargs):
        self.linked_transaction_id = linked_transaction_id
        self.active = active
        self.version = version
        super(Link, self).__init__(*args, **kwargs)


class Party(AMaaSModel):

    def __init__(self, party_id, active=True, version=1, *args, **kwargs):
        self.party_id = party_id
        self.active = active
        self.version = version
        super(Party, self).__init__(*args, **kwargs)



import requests

from amaascore.books.utils import json_to_book
from amaascore.config import ENDPOINTS
from amaascore.core.interface import Interface


class BooksInterface(Interface):

    def __init__(self):
        endpoint = ENDPOINTS.get('books')
        super(BooksInterface, self).__init__(endpoint=endpoint)

    def new(self, book):
        url = self.endpoint + '/books'
        response = requests.post(url, json=book.to_interface())
        if response.ok:
            book = json_to_book(response.json())
            return book
        else:
            print("HANDLE THIS PROPERLY")
            print(response.content)

    def amend(self, book):
        url = '%s/books/%s/%s' % (self.endpoint, book.asset_manager_id, book.book_id)
        response = requests.put(url, json=book.to_interface())
        if response.ok:
            book = json_to_book(response.json())
            return book
        else:
            print("HANDLE THIS PROPERLY")
            print(response.content)

    def retrieve(self, asset_manager_id, book_id):
        url = '%s/books/%s/%s' % (self.endpoint, asset_manager_id, book_id)
        response = requests.get(url)
        if response.ok:
            return json_to_book(response.json())
        else:
            print("HANDLE THIS PROPERLY")
            print(response.content)

    def retire(self, asset_manager_id, book_id):
        url = '%s/books/%s/%s' % (self.endpoint, asset_manager_id, book_id)
        response = requests.delete(url)
        if response.ok:
            print("DO SOMETHING?")
        else:
            print("HANDLE THIS PROPERLY")
            print(response.content)

    def search(self, asset_manager_ids=None, book_ids=None):
        search_params = {}
        # Potentially roll this into a loop through args rather than explicitly named - depends on additional validation
        if asset_manager_ids:
            search_params['asset_manager_ids'] = asset_manager_ids
        if book_ids:
            search_params['book_ids'] = book_ids
        url = self.endpoint + '/books'
        response = requests.get(url, params=search_params)
        if response.ok:
            books = [json_to_book(json_book) for json_book in response.json()]
            return books
        else:
            print("HANDLE THIS PROPERLY")
            print(response.content)

    def books_by_asset_manager(self, asset_manager_id):
        url = '%s/books/%s' % (self.endpoint, asset_manager_id)
        response = requests.get(url)
        if response.ok:
            books = [json_to_book(json_book) for json_book in response.json()]
            return books
        else:
            print("HANDLE THIS PROPERLY")
            print(response.content)


from amaascore.parties.company import Company


class Broker(Company):

    def __init__(self, asset_manager_id, party_id, base_currency=None, description='', party_status='Active',
                 addresses={}, emails={}, links={}, references={},
                 *args, **kwargs):
        super(Broker, self).__init__(asset_manager_id=asset_manager_id, party_id=party_id, base_currency=base_currency,
                                     description=description, party_status=party_status,
                                     addresses=addresses, emails=emails, links=links,
                                     references=references, *args, **kwargs)

# TODO - Add markets covered, products covered



from amaascore.assets.asset import Asset


class Currency(Asset):

    def __init__(self, asset_id, deliverable, asset_status='Active', minor_unit_places=2, description='',
                 country_id=None, *args, **kwargs):
        self.asset_class = 'Currency'
        self.deliverable = deliverable
        self.minor_unit_places = minor_unit_places
        super(Currency, self).__init__(asset_manager_id=0, asset_id=asset_id, fungible=True,
                                       asset_class=self.asset_class, asset_status=asset_status, description=description,
                                       country_id=country_id, venue_id=None, maturity_date=None,
                                       *args, **kwargs)



from datetime import date

from amaascore.assets.asset import Asset


class Index(Asset):

    def __init__(self, asset_id, asset_manager_id, asset_issuer_id=None, asset_status='Active', country_id=None,
                 issue_date=date.min, description='', links={}, references={}, *args, **kwargs):
        self.asset_class = 'Index'
        super(Index, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id, fungible=True,
                                    asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                    country_id=country_id, description=description, links=links, references=references,
                                    issue_date=issue_date, *args, **kwargs)



from datetime import date
from decimal import Decimal

from amaascore.assets.future_option import FutureOption


class BondFutureOption(FutureOption):

    def __init__(self, asset_manager_id, asset_id, option_type, option_style, strike, underlying_asset_id,
                 asset_issuer_id=None, asset_status='Active', issue_date=date.min, expiry_date=date.max,
                 description='', country_id=None, venue_id=None, links={}, references={}, *args, **kwargs):
        """

        :param asset_manager_id: The asset manager who owns the data for this BondFutureOption
        :param asset_id: The owning asset manager's unique identifier for this BondFutureOption
        :param option_type: Whether this is a put option or a call option
        :param option_style: The style of the option - e.g. American, European
        :param strike: The option strike price
        :param underlying_asset_id: The owning asset manager's identifier for the asset which underlies this option
        :param asset_issuer_id: The owning asset manager's identifier for the issuer of this option
        :param asset_status: The status of this BondFutureOption
        :param issue_date: The date on which this BondFutureOption was created
        :param expiry_date: The date on which this BondFutureOption will expire
        :param description: A human-readable description of this BondFutureOption
        :param country_id: The ISO 3166-1 alpha-3 code for the country that this BondFutureOption was issued in
        :param venue_id: The ISO-10383 identifier for the venue on which this BondFutureOption trades
        :param links: Links to other relevant assets
        :param references: A dictionary of additional References for this BondFutureOption
        :param args:
        :param kwargs:
        """
        super(BondFutureOption, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id,
                                               asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                               description=description, country_id=country_id, venue_id=venue_id,
                                               links=links, references=references, issue_date=issue_date,
                                               expiry_date=expiry_date, option_style=option_style, strike=strike,
                                               option_type=option_type, underlying_asset_id=underlying_asset_id,
                                               *args, **kwargs)



from datetime import date, datetime
from dateutil import parser
from decimal import Decimal

from amaascore.assets.asset import Asset


class Derivative(Asset):

    def __init__(self, asset_manager_id, asset_id, premium=None, maturity_date=None, asset_issuer_id=None,
                 asset_status='Active', description='', country_id=None, venue_id=None, issue_date=None,
                 links={}, references={},
                 *args, **kwargs):
        if not hasattr(self, 'asset_class'):  # A more specific child class may have already set this
            self.asset_class = 'Derivative'
        self.premium = premium
        super(Derivative, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id, fungible=False,
                                         asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                         description=description, country_id=country_id, venue_id=venue_id,
                                         maturity_date=maturity_date, links=links, references=references,
                                         issue_date=issue_date,
                                         *args, **kwargs)



# TODO - Support multi-language errors
ERROR_LOOKUP = {'address_invalid': 'Invalid addresses attribute: %s. Party ID %s for Asset Manager %s',
                'address_primary': 'Must set exactly one address as primary. Party ID %s for Asset Manager %s',
                'email_invalid': 'Invalid emails attribute: %s. Party ID %s for Asset Manager %s',
                'email_primary': 'Must set exactly one email as primary. Party ID %s for Asset Manager %s',
                'email_address_invalid': 'Invalid email(s): %s. Party ID %s for Asset Manager %s',
                'amend_missing_previous': 'Cannot find party to amend: ID %s for Asset Manager %s',
                'amend_missing_attribute': 'Partial amend failed for Asset Manager: %s on party: %s - '
                                           'Attribute: %s does not exist',
                'deactivate_missing_previous': 'Cannot Deactivate Party - Cannot Find ID: %s for Asset Manager: %s',
                'party_status_invalid': 'Invalid party status %s. Party ID: %s for Asset Manager: %s',
                'transaction_action_invalid': 'Invalid transaction action %s. Transaction ID: %s for Asset Manager: %s',
                'transaction_status_invalid': 'Invalid transaction status %s. Transaction ID: %s for Asset Manager: %s',
                'transaction_type_invalid': 'Invalid transaction type %s. Transaction ID: %s for Asset Manager: %s',
                'transaction_link_not_found': 'Cannot remove link - not found'}
